@@ -12,11 +12,15 @@
 
 #include "serialization_utils.h"
 
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_stdstream.hpp"
+
+#include <array>
 #include <istream>
 #include <ostream>
+#include <sstream>
 
 #include "misc_util.h"
-#include "mptBufferIO.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -26,13 +30,15 @@ namespace srlztn
 {
 
 
-//#define SSB_LOGGING
+#ifdef MPT_ALL_LOGGING
+#define SSB_LOGGING
+#endif
 
 
 #ifdef SSB_LOGGING
-#define SSB_LOG(x) Log(x)
+#define SSB_LOG(x) MPT_LOG_GLOBAL(LogDebug, "serialization", x)
 #else
-#define SSB_LOG(x) MPT_DO { } MPT_WHILE_0
+#define SSB_LOG(x) do { } while(0)
 #endif
 
 
@@ -65,7 +71,7 @@ bool ID::IsPrintable() const
 static void WriteAdaptive12String(std::ostream& oStrm, const std::string& str)
 {
 	uint16 s = static_cast<uint16>(str.size());
-	LimitMax(s, uint16(uint16_max / 2));
+	LimitMax(s, uint16(std::numeric_limits<uint16>::max() / 2));
 	mpt::IO::WriteAdaptiveInt16LE(oStrm, s);
 	oStrm.write(str.c_str(), s);
 }
@@ -73,7 +79,7 @@ static void WriteAdaptive12String(std::ostream& oStrm, const std::string& str)
 
 void WriteItemString(std::ostream& oStrm, const std::string &str)
 {
-	uint32 id = static_cast<uint32>(std::min<std::size_t>(str.size(), (uint32_max >> 4))) << 4;
+	uint32 id = static_cast<uint32>(std::min(str.size(), static_cast<std::size_t>((std::numeric_limits<uint32>::max() >> 4)))) << 4;
 	id |= 12; // 12 == 1100b
 	Binarywrite<uint32>(oStrm, id);
 	id >>= 4;
@@ -91,7 +97,7 @@ void ReadItemString(std::istream& iStrm, std::string& str, const DataSize)
 	const uint8 nSizeBytes = (id & 12) >> 2; // 12 == 1100b
 	if (nSizeBytes > 0)
 	{
-		uint8 bytes = std::min<uint8>(3, nSizeBytes);
+		uint8 bytes = std::min(uint8(3), nSizeBytes);
 		uint8 v2 = 0;
 		uint8 v3 = 0;
 		uint8 v4 = 0;
@@ -102,7 +108,7 @@ void ReadItemString(std::istream& iStrm, std::string& str, const DataSize)
 		id |= (v2 << 8) | (v3 << 16) | (v4 << 24);
 	}
 	// Limit to 1 MB.
-	str.resize(std::min<std::size_t>(id >> 4, 1000000));
+	str.resize(std::min(id >> 4, uint32(1000000)));
 	for(size_t i = 0; i < str.size(); i++)
 		iStrm.read(&str[i], 1);
 
@@ -116,7 +122,7 @@ mpt::ustring ID::AsString() const
 {
 	if(IsPrintable())
 	{
-		return mpt::ToUnicode(mpt::CharsetISO8859_1, m_ID);
+		return mpt::ToUnicode(mpt::Charset::ISO8859_1, m_ID);
 	}
 	if(m_ID.length() > 8)
 	{
@@ -132,26 +138,6 @@ mpt::ustring ID::AsString() const
 const char Ssb::s_EntryID[3] = {'2','2','8'};
 
 
-#ifdef SSB_LOGGING
-static const MPT_UCHAR_TYPE tstrWriteHeader[] = UL_("Write header with ID = %1\n");
-static const MPT_UCHAR_TYPE tstrWriteProgress[] = UL_("Wrote entry: {num, id, rpos, size} = {%1, %2, %3, %4}\n");
-static const MPT_UCHAR_TYPE tstrWritingMap[] = UL_("Writing map to rpos: %1\n");
-static const MPT_UCHAR_TYPE tstrMapEntryWrite[] = UL_("Writing map entry: id=%1, rpos=%2, size=%3\n");
-static const MPT_UCHAR_TYPE strWriteNote[] = UL_("Write note: ");
-static const MPT_UCHAR_TYPE tstrEndOfStream[] = UL_("End of stream(rpos): %1\n");
-
-static const MPT_UCHAR_TYPE tstrReadingHeader[] = UL_("Read header with expected ID = %1\n");
-static const MPT_UCHAR_TYPE strNoMapInFile[] = UL_("No map in the file.\n");
-static const MPT_UCHAR_TYPE strIdMismatch[] = UL_("ID mismatch, terminating read.\n");
-static const MPT_UCHAR_TYPE strIdMatch[] = UL_("ID match, continuing reading.\n");
-static const MPT_UCHAR_TYPE tstrReadingMap[] = UL_("Reading map from rpos: %1\n");
-static const MPT_UCHAR_TYPE tstrEndOfMap[] = UL_("End of map(rpos): %1\n");
-static const MPT_UCHAR_TYPE tstrReadProgress[] = UL_("Read entry: {num, id, rpos, size, desc} = {%1, %2, %3, %4, %5}\n");
-static const MPT_UCHAR_TYPE tstrNoEntryFound[] = UL_("No entry with id %1 found.\n");
-static const MPT_UCHAR_TYPE strReadNote[] = UL_("Read note: ");
-#endif
-
-
 Ssb::Ssb()
 	: m_Status(SNT_NONE)
 	, m_nFixedEntrySize(0)
@@ -190,19 +176,19 @@ SsbRead::SsbRead(std::istream& is)
 void SsbWrite::AddWriteNote(const SsbStatus s)
 {
 	m_Status |= s;
-	SSB_LOG(mpt::format(U_("%1: 0x%2\n"))(strWriteNote, mpt::ufmt::hex(s)));
+	SSB_LOG(MPT_UFORMAT("{}: 0x{}")(U_("Write note: "), mpt::ufmt::hex(s)));
 }
 
 void SsbRead::AddReadNote(const SsbStatus s)
 {
 	m_Status |= s;
-	SSB_LOG(mpt::format(U_("%1: 0x%2\n"))(strReadNote, mpt::ufmt::hex(s)));
+	SSB_LOG(MPT_UFORMAT("{}: 0x{}")(U_("Read note: "), mpt::ufmt::hex(s)));
 }
 
 void SsbRead::AddReadNote(const ReadEntry* const pRe, const NumType nNum)
 {
 	m_Status |= SNT_PROGRESS;
-	SSB_LOG(mpt::format(mpt::ustring(tstrReadProgress))(
+	SSB_LOG(MPT_UFORMAT("Read entry: {{num, id, rpos, size, desc}} = {{{}, {}, {}, {}, {}}}")(
 				 nNum,
 				 (pRe && pRe->nIdLength < 30 && m_Idarray.size() > 0) ?  ID(&m_Idarray[pRe->nIdpos], pRe->nIdLength).AsString() : U_(""),
 				 (pRe) ? pRe->rposStart : 0,
@@ -218,7 +204,7 @@ void SsbRead::AddReadNote(const ReadEntry* const pRe, const NumType nNum)
 void SsbWrite::AddWriteNote(const ID &id, const NumType nEntryNum, const DataSize nBytecount, const RposType rposStart)
 {
 	m_Status |= SNT_PROGRESS;
-	SSB_LOG(mpt::format(mpt::ustring(tstrWriteProgress))(nEntryNum, id.AsString(), rposStart, nBytecount));
+	SSB_LOG(MPT_UFORMAT("Wrote entry: {{num, id, rpos, size}} = {{{}, {}, {}, {}}}")(nEntryNum, id.AsString(), rposStart, nBytecount));
 #ifndef SSB_LOGGING
 	MPT_UNREFERENCED_PARAMETER(id);
 	MPT_UNREFERENCED_PARAMETER(nEntryNum);
@@ -241,12 +227,12 @@ void SsbWrite::WriteMapItem(const ID &id,
 						const DataSize& nDatasize,
 						const char* pszDesc)
 {
-	SSB_LOG(mpt::format(mpt::ustring(tstrMapEntryWrite))(
+	SSB_LOG(MPT_UFORMAT("Writing map entry: id={}, rpos={}, size={}")(
 					(id.GetSize() > 0) ? id.AsString() : U_(""),
 					rposDataStart,
 					nDatasize));
 
-	mpt::ostringstream mapStream;
+	std::ostringstream mapStream;
 
 	if(m_nIdbytes > 0)
 	{
@@ -275,7 +261,7 @@ void SsbWrite::WriteMapItem(const ID &id,
 void SsbWrite::IncrementWriteCounter()
 {
 	m_nCounter++;
-	if (m_nCounter >= (uint16_max >> 2))
+	if(m_nCounter >= static_cast<uint16>(std::numeric_limits<uint16>::max() >> 2))
 	{
 		FinishWrite();
 		AddWriteNote(SNW_MAX_WRITE_COUNT_REACHED);
@@ -285,7 +271,7 @@ void SsbWrite::IncrementWriteCounter()
 
 void SsbWrite::BeginWrite(const ID &id, const uint64& nVersion)
 {
-	SSB_LOG(mpt::format(mpt::ustring(tstrWriteHeader))(id.AsString()));
+	SSB_LOG(MPT_UFORMAT("Write header with ID = {}")(id.AsString()));
 
 	ResetWritestatus();
 
@@ -371,7 +357,7 @@ SsbRead::ReadRv SsbRead::OnReadEntry(const ReadEntry* pE, const ID &id, const Po
 	}
 	else // Entry not found.
 	{
-		SSB_LOG(mpt::format(mpt::ustring(tstrNoEntryFound))(id.AsString()));
+		SSB_LOG(MPT_UFORMAT("No entry with id {} found.")(id.AsString()));
 #ifndef SSB_LOGGING
 		MPT_UNREFERENCED_PARAMETER(id);
 #endif
@@ -386,8 +372,11 @@ void SsbWrite::OnWroteItem(const ID &id, const Postype& posBeforeWrite)
 {
 	const Offtype nRawEntrySize = oStrm.tellp() - posBeforeWrite;
 
-	if (nRawEntrySize < 0 || static_cast<uint64>(nRawEntrySize) > std::numeric_limits<DataSize>::max())
-		{ AddWriteNote(SNW_INSUFFICIENT_DATASIZETYPE); return; }
+	MPT_MAYBE_CONSTANT_IF(nRawEntrySize < 0 || static_cast<uint64>(nRawEntrySize) > std::numeric_limits<DataSize>::max())
+	{
+		AddWriteNote(SNW_INSUFFICIENT_DATASIZETYPE);
+		return;
+	}
 
 	if(GetFlag(RwfRMapHasSize) && (nRawEntrySize < 0 || static_cast<uint64>(nRawEntrySize) > (std::numeric_limits<DataSize>::max() >> 2)))
 		{ AddWriteNote(SNW_DATASIZETYPE_OVERFLOW); return; }
@@ -416,7 +405,7 @@ void SsbWrite::OnWroteItem(const ID &id, const Postype& posBeforeWrite)
 
 void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 {
-	SSB_LOG(mpt::format(mpt::ustring(tstrReadingHeader))(id.AsString()));
+	SSB_LOG(MPT_UFORMAT("Read header with expected ID = {}")(id.AsString()));
 
 	ResetReadstatus();
 
@@ -439,23 +428,23 @@ void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 	// Compare IDs.
 	uint8 storedIdLen = 0;
 	Binaryread<uint8>(iStrm, storedIdLen);
-	char storedIdBuf[256];
-	Clear(storedIdBuf);
+	std::array<char, 256> storedIdBuf;
+	storedIdBuf = {};
 	if(storedIdLen > 0)
 	{
-		iStrm.read(storedIdBuf, storedIdLen);
+		iStrm.read(storedIdBuf.data(), storedIdLen);
 	}
-	if(!(id == ID(storedIdBuf, storedIdLen)))
+	if(!(id == ID(storedIdBuf.data(), storedIdLen)))
 	{
 		AddReadNote(SNR_OBJECTCLASS_IDMISMATCH);
 	}
 	if ((m_Status & SNT_FAILURE) != 0)
 	{
-		SSB_LOG(mpt::ustring(strIdMismatch));
+		SSB_LOG(U_("ID mismatch, terminating read."));
 		return;
 	}
 
-	SSB_LOG(mpt::ustring(strIdMatch));
+	SSB_LOG(U_("ID match, continuing reading."));
 	
 	// Header
 	uint8 tempU8;
@@ -521,7 +510,7 @@ void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 	
 	if (GetFlag(RwfRwHasMap) == false)
 	{
-		SSB_LOG(mpt::ustring(strNoMapInFile));
+		SSB_LOG(U_("No map in the file."));
 	}
 
 	if (Testbit(flagbyte, 2)) // Object description?
@@ -555,8 +544,11 @@ void SsbRead::BeginRead(const ID &id, const uint64& nVersion)
 
 	const Offtype rawEndOfHdrData = iStrm.tellg() - m_posStart;
 
-	if (rawEndOfHdrData < 0 || static_cast<uint64>(rawEndOfHdrData) > std::numeric_limits<RposType>::max())
-		{ AddReadNote(SNR_INSUFFICIENT_RPOSTYPE); return; }
+	MPT_MAYBE_CONSTANT_IF(rawEndOfHdrData < 0 || static_cast<uint64>(rawEndOfHdrData) > std::numeric_limits<RposType>::max())
+	{
+		AddReadNote(SNR_INSUFFICIENT_RPOSTYPE);
+		return;
+	}
 
 	m_rposEndofHdrData = static_cast<RposType>(rawEndOfHdrData);
 	m_rposMapBegin = (GetFlag(RwfRwHasMap)) ? static_cast<RposType>(tempU64) : m_rposEndofHdrData;
@@ -577,7 +569,7 @@ void SsbRead::CacheMap()
 		if(iStrm.fail())
 			{ AddReadNote(SNR_BADSTREAM_AFTER_MAPHEADERSEEK); return; }
 
-		SSB_LOG(mpt::format(mpt::ustring(tstrReadingMap))(m_rposMapBegin));
+		SSB_LOG(MPT_UFORMAT("Reading map from rpos: {}")(m_rposMapBegin));
 
 		mapData.resize(m_nReadEntrycount);
 		m_Idarray.reserve(m_nReadEntrycount * 4);
@@ -639,7 +631,7 @@ void SsbRead::CacheMap()
 			}
 		}
 		m_posMapEnd = iStrm.tellg();
-		SSB_LOG(mpt::format(mpt::ustring(tstrEndOfMap))(m_posMapEnd - m_posStart));
+		SSB_LOG(MPT_UFORMAT("End of map(rpos): {}")(m_posMapEnd - m_posStart));
 	}
 
 	SetFlag(RwfRMapCached, true);
@@ -692,7 +684,7 @@ void SsbWrite::FinishWrite()
 		
 	Postype posMapStart = oStrm.tellp();
 
-	SSB_LOG(mpt::format(mpt::ustring(tstrWritingMap))(posMapStart - m_posStart));
+	SSB_LOG(MPT_UFORMAT("Writing map to rpos: {}")(posMapStart - m_posStart));
 
 	if (GetFlag(RwfRwHasMap)) //Write map
 	{
@@ -718,9 +710,9 @@ void SsbWrite::FinishWrite()
 	}
 
 	// Seek to end.
-	oStrm.seekp(std::max<Postype>(posMapEnd, posDataEnd)); 
+	oStrm.seekp(std::max(posMapEnd, posDataEnd)); 
 
-	SSB_LOG(mpt::format(mpt::ustring(tstrEndOfStream))(oStrm.tellp() - m_posStart));
+	SSB_LOG(MPT_UFORMAT("End of stream(rpos): {}")(oStrm.tellp() - m_posStart));
 }
 
 } // namespace srlztn 
