@@ -127,12 +127,12 @@
 
 /*! \defgroup libopenmpt_cpp libopenmpt C++ */
 
+namespace openmpt {
+
 /*! \addtogroup libopenmpt_cpp
   @{
 */
 
-namespace openmpt {
-
 #if defined(_MSC_VER)
 #pragma warning(push)
 #pragma warning(disable:4275)
@@ -1132,7 +1132,7 @@ public:
 	  \param value The value that should be set.
 	  \throws openmpt::exception Throws an exception derived from openmpt::exception in case the value is not sensible (e.g. negative tempo factor) or under the circumstances outlined in openmpt::module::get_ctls.
 	  \sa openmpt::module::get_ctls
-	  \deprecated Please use openmpt::module::ctl_set_bool(), openmpt::module::ctl_set_int(), openmpt::module::ctl_set_floatingpoint(), or openmpt::module::ctl_set_string().
+	  \deprecated Please use openmpt::module::ctl_set_boolean(), openmpt::module::ctl_set_integer(), openmpt::module::ctl_set_floatingpoint(), or openmpt::module::ctl_set_text().
 	*/
 	LIBOPENMPT_ATTR_DEPRECATED void ctl_set( const std::string & ctl, const std::string & value );
 	//! Set ctl boolean value
@@ -1176,10 +1176,10 @@ public:
 
 }; // class module
 
-} // namespace openmpt
-
 /*!
   @}
 */
 
+} // namespace openmpt
+
 #endif // LIBOPENMPT_HPP
