@@ -14,14 +14,13 @@
 #include "libopenmpt_impl.hpp"
 #include "libopenmpt_ext.hpp"
 
-using namespace OpenMPT;
-
 namespace openmpt {
 
 class module_ext_impl
 	: public module_impl
 	, public ext::pattern_vis
 	, public ext::interactive
+	, public ext::interactive2
 
 
 
@@ -33,8 +32,10 @@ class module_ext_impl
 public:
 	module_ext_impl( callback_stream_wrapper stream, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( std::istream & stream, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
+	module_ext_impl( const std::vector<std::byte> & data, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( const std::vector<std::uint8_t> & data, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( const std::vector<char> & data, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
+	module_ext_impl( const std::byte * data, std::size_t size, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( const std::uint8_t * data, std::size_t size, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( const char * data, std::size_t size, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
 	module_ext_impl( const void * data, std::size_t size, std::unique_ptr<log_interface> log, const std::map< std::string, std::string > & ctls );
@@ -99,11 +100,20 @@ public:
 
 	void stop_note( std::int32_t channel ) override;
 
+	void note_off( std::int32_t channel ) override;
+
+	void note_fade( std::int32_t channel ) override;
+
+	void set_channel_panning( std::int32_t channel, double panning ) override;
+
+	double get_channel_panning( std::int32_t channel ) override;
+
+	void set_note_finetune( std::int32_t channel, double finetune ) override;
+
+	double get_note_finetune( std::int32_t channel ) override;
 
 	/* add stuff here */
 
-
-
 }; // class module_ext_impl
 
 } // namespace openmpt
