@@ -8,7 +8,7 @@
  */
 
 static const char * const license =
-"Copyright (c) 2004-2022, OpenMPT Project Developers and Contributors" "\n"
+"Copyright (c) 2004-2023, OpenMPT Project Developers and Contributors" "\n"
 "Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 "All rights reserved." "\n"
 "" "\n"
@@ -464,7 +464,7 @@ static std::string seconds_to_string( double time ) {
 
 static void show_info( std::ostream & log, bool verbose ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << ", libopenmpt " << openmpt::string::get( "library_version" ) << " (" << "OpenMPT " << openmpt::string::get( "core_version" ) << ")" << std::endl;
-	log << "Copyright (c) 2013-2022 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2023 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
 	if ( !verbose ) {
 		log << std::endl;
 		return;
@@ -541,7 +541,7 @@ static void show_info( std::ostream & log, bool verbose ) {
 static void show_man_version( textout & log ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << std::endl;
 	log << std::endl;
-	log << "Copyright (c) 2013-2022 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2023 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
 }
 
 static void show_short_version( textout & log ) {
@@ -587,6 +587,34 @@ static std::string get_device_string( const std::string & device ) {
 	return device;
 }
 
+static void show_help_keyboard( textout & log, bool man_version = false ) {
+	if ( !man_version ) {
+		show_info( log, false );
+	}
+	log << "Keyboard hotkeys (use 'openmpt123 --ui'):" << std::endl;
+	log << std::endl;
+	log << " [q]      quit" << std::endl;
+	log << " [ ]      pause / unpause" << std::endl;
+	log << " [N]      skip 10 files backward" << std::endl;
+	log << " [n]      prev file" << std::endl;
+	log << " [m]      next file" << std::endl;
+	log << " [M]      skip 10 files forward" << std::endl;
+	log << " [h]      seek 10 seconds backward" << std::endl;
+	log << " [j]      seek 1 seconds backward" << std::endl;
+	log << " [k]      seek 1 seconds forward" << std::endl;
+	log << " [l]      seek 10 seconds forward" << std::endl;
+	log << " [u]|[i]  +/- tempo" << std::endl;
+	log << " [o]|[p]  +/- pitch" << std::endl;
+	log << " [3]|[4]  +/- gain" << std::endl;
+	log << " [5]|[6]  +/- stereo separation" << std::endl;
+	log << " [7]|[8]  +/- filter taps" << std::endl;
+	log << " [9]|[0]  +/- volume ramping" << std::endl;
+	log << std::endl;
+	if ( !man_version ) {
+		log.writeout();
+	}
+}
+
 static void show_help( textout & log, bool with_info = true, bool longhelp = false, bool man_version = false, const std::string & message = std::string() ) {
 	if ( with_info ) {
 		show_info( log, false );
@@ -600,6 +628,7 @@ static void show_help( textout & log, bool with_info = true, bool longhelp = fal
 		}
 		if ( man_version ) {
 			log << "Options:" << std::endl;
+			log << std::endl;
 		}
 		log << " -h, --help                 Show help" << std::endl;
 		log << "     --help-keyboard        Show keyboard hotkeys in ui mode" << std::endl;
@@ -685,6 +714,8 @@ static void show_help( textout & log, bool with_info = true, bool longhelp = fal
 				log << extension;
 			}
 			log << std::endl;
+		} else {
+			show_help_keyboard( log, true );
 		}
 	}
 
@@ -697,30 +728,6 @@ static void show_help( textout & log, bool with_info = true, bool longhelp = fal
 	log.writeout();
 }
 
-static void show_help_keyboard( textout & log ) {
-	show_info( log, false );
-	log << "Keyboard hotkeys (use 'openmpt123 --ui'):" << std::endl;
-	log << std::endl;
-	log << " [q]     quit" << std::endl;
-	log << " [ ]     pause / unpause" << std::endl;
-	log << " [N]     skip 10 files backward" << std::endl;
-	log << " [n]     prev file" << std::endl;
-	log << " [m]     next file" << std::endl;
-	log << " [M]     skip 10 files forward" << std::endl;
-	log << " [h]     seek 10 seconds backward" << std::endl;
-	log << " [j]     seek 1 seconds backward" << std::endl;
-	log << " [k]     seek 1 seconds forward" << std::endl;
-	log << " [l]     seek 10 seconds forward" << std::endl;
-	log << " [u]|[i] +/- tempo" << std::endl;
-	log << " [o]|[p] +/- pitch" << std::endl;
-	log << " [3]|[4] +/- gain" << std::endl;
-	log << " [5]|[6] +/- stereo separation" << std::endl;
-	log << " [7]|[8] +/- filter taps" << std::endl;
-	log << " [9]|[0] +/- volume ramping" << std::endl;
-	log << std::endl;
-	log.writeout();
-}
-
 
 template < typename Tmod >
 static void apply_mod_settings( commandlineflags & flags, Tmod & mod ) {
@@ -1057,6 +1064,7 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 	
 	if ( multiline ) {
 		lines += 1;
+		// cppcheck-suppress identicalInnerCondition
 		if ( flags.show_ui ) {
 			lines += 1;
 		}
@@ -1174,7 +1182,7 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 				cpu /= ( static_cast<double>( count ) ) / static_cast<double>( flags.samplerate );
 				double mix = ( static_cast<double>( count ) ) / static_cast<double>( flags.samplerate );
 				cpu_smooth = ( 1.0 - mix ) * cpu_smooth + mix * cpu;
-				sprintf( cpu_str, "%.2f%%", cpu_smooth * 100.0 );
+				std::snprintf( cpu_str, 64, "%.2f%%", cpu_smooth * 100.0 );
 			}
 		}
 
@@ -2528,7 +2536,7 @@ static int main( int argc, char * argv [] ) {
 } // namespace openmpt123
 
 #if defined(WIN32) && defined(UNICODE)
-#if defined(__GNUC__)
+#if defined(__GNUC__) || (defined(__clang__) && !defined(_MSC_VER))
 // mingw64 does only default to special C linkage for "main", but not for "wmain".
 extern "C" int wmain( int wargc, wchar_t * wargv [] );
 extern "C"
