@@ -32,7 +32,14 @@
 #endif
 #ifdef UNICODE
 #ifndef _UNICODE
+#if defined(__clang__)
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
+#endif
 #define _UNICODE
+#if defined(__clang__)
+#pragma clang diagnostic pop
+#endif
 #endif
 #endif
 #endif // WIN32
