@@ -646,6 +646,7 @@ bool CDLSBank::FindAndExtract(CSoundFile &sndFile, const INSTRUMENTINDEX ins, co
 	const uint32 program = (pIns->nMidiProgram != 0) ? pIns->nMidiProgram - 1 : 0;
 	const uint32 key = isDrum ? (pIns->nMidiDrumKey & 0x7F) : 0xFF;
 	if(FindInstrument(isDrum, (pIns->wMidiBank - 1) & 0x3FFF, program, key, &dlsIns)
+		|| FindInstrument(isDrum, (pIns->wMidiBank - 1) & 0x3F80, program, key, &dlsIns)
 		|| FindInstrument(isDrum, 0xFFFF, isDrum ? 0xFF : program, key, &dlsIns))
 	{
 		if(key < 0x80) drumRgn = GetRegionFromKey(dlsIns, key);
@@ -1123,7 +1124,7 @@ bool CDLSBank::ConvertSF2ToDLS(SF2LoaderInfo &sf2info)
 		}
 		// Load Instrument Bags
 		dlsIns.Regions.clear();
-		for(const auto [nInstrNdx, keyRange] : instruments)
+		for(const auto & [nInstrNdx, keyRange] : instruments)
 		{
 			if(nInstrNdx >= numInsts)
 				continue;
@@ -2009,9 +2010,9 @@ bool CDLSBank::ExtractInstrument(CSoundFile &sndFile, INSTRUMENTINDEX nInstr, ui
 				} else
 				{
 					SmpLength len = std::min(dwLen / 2u, sampleCopy.nLength);
-					const int16 *src = reinterpret_cast<int16 *>(pWaveForm.data());
+					const std::byte *src = mpt::byte_cast<const std::byte *>(pWaveForm.data());
 					int16 *dst = sampleCopy.sample16() + offsetNew;
-					CopySample<SC::ConversionChain<SC::Convert<int16, int16>, SC::DecodeIdentity<int16>>>(dst, len, 2, src, pWaveForm.size(), 1);
+					CopySample<SC::ConversionChain<SC::Convert<int16, int16>, SC::DecodeInt16<0, littleEndian16>>>(dst, len, 2, src, pWaveForm.size(), 1);
 				}
 				sample.FreeSample();
 				sample = sampleCopy;
