@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 #include "../soundlib/ModInstrument.h"
 #include "../soundlib/ModSample.h"
@@ -63,7 +63,7 @@ struct ITFileHeader
 	uint8le  pwd;				// Pitch Wheel Depth
 	uint16le msglength;			// Length of Song Message
 	uint32le msgoffset;			// Offset of Song Message in File (IT crops message after first null)
-	uint32le reserved;			// Some IT versions save an edit timer here. ChibiTracker writes "CHBI" here. OpenMPT writes "OMPT" here in some cases, see Load_it.cpp
+	uint32le reserved;			// Some IT versions save an edit timer here. ChibiTracker writes "CHBI" here. OpenMPT and Schism Tracker save extended version information here.
 	uint8le  chnpan[64];		// Initial Channel Panning
 	uint8le  chnvol[64];		// Initial Channel Volume
 };
@@ -275,9 +275,9 @@ struct FileHistory;
 // IT Header extension: Save history
 struct ITHistoryStruct
 {
-	uint16le fatdate;	// DOS / FAT date when the file was opened / created in the editor. For details, read http://msdn.microsoft.com/en-us/library/ms724247(VS.85).aspx
-	uint16le fattime;	// DOS / FAT time when the file was opened / created in the editor.
-	uint32le runtime;	// The time how long the file was open in the editor, in 1/18.2th seconds. (= ticks of the DOS timer)
+	uint16le fatdate;  // DOS / FAT date when the file was opened / created in the editor. For details, read https://docs.microsoft.com/de-de/windows/win32/api/winbase/nf-winbase-dosdatetimetofiletime
+	uint16le fattime;  // DOS / FAT time when the file was opened / created in the editor.
+	uint32le runtime;  // The time how long the file was open in the editor, in 1/18.2th seconds. (= ticks of the DOS timer)
 
 	// Convert an ITHistoryStruct to OpenMPT's internal edit history representation
 	void ConvertToMPT(FileHistory &mptHistory) const;
@@ -307,14 +307,17 @@ enum IT_ReaderBitMasks
 template<int32 y, int32 m, int32 d>
 struct SchismVersionFromDate
 {
-	enum : int32 { mm = (m + 9) % 12 };
-	enum : int32 { yy = y - mm / 10 };
-	enum : int32 { date = yy * 365 + yy / 4 - yy / 100 + yy / 400 + (mm * 306 + 5) / 10 + (d - 1) };
+private:
+	static constexpr int32 mm = (m + 9) % 12;
+	static constexpr int32 yy = y - mm / 10;
 
-	static constexpr int32 Version(const int32 trackerID = 0x1000)
-	{
-		return trackerID + 0x0050 + date - SchismVersionFromDate<2009, 10, 31>::date;
-	}
+public:
+	static constexpr int32 date = yy * 365 + yy / 4 - yy / 100 + yy / 400 + (mm * 306 + 5) / 10 + (d - 1);
 };
 
+inline constexpr int32 SchismTrackerEpoch = SchismVersionFromDate<2009, 10, 31>::date;
+
+
+uint32 DecodeITEditTimer(uint16 cwtv, uint32 editTime);
+
 OPENMPT_NAMESPACE_END
