@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 OPENMPT_NAMESPACE_BEGIN
 
@@ -42,10 +42,12 @@ struct ModSample
 	uint8  rootNote;						// For multisample import
 
 	//char name[MAX_SAMPLENAME];			// Maybe it would be nicer to have sample names here, but that would require some refactoring.
-	char filename[MAX_SAMPLEFILENAME];
+	mpt::charbuf<MAX_SAMPLEFILENAME> filename;
+	std::string GetFilename() const { return filename; }
+
 	union
 	{
-		SmpLength cues[9];
+		std::array<SmpLength, 9> cues;
 		OPLPatch adlib;
 	};
 
@@ -69,13 +71,13 @@ struct ModSample
 	{
 		return pData.pSample;
 	}
-	MPT_FORCEINLINE const mpt::byte *sampleb() const noexcept
+	MPT_FORCEINLINE const std::byte *sampleb() const noexcept
 	{
-		return mpt::void_cast<const mpt::byte*>(pData.pSample);
+		return mpt::void_cast<const std::byte*>(pData.pSample);
 	}
-	MPT_FORCEINLINE mpt::byte *sampleb() noexcept
+	MPT_FORCEINLINE std::byte *sampleb() noexcept
 	{
-		return mpt::void_cast<mpt::byte*>(pData.pSample);
+		return mpt::void_cast<std::byte*>(pData.pSample);
 	}
 	MPT_FORCEINLINE const int8 *sample8() const noexcept
 	{
@@ -120,6 +122,9 @@ struct ModSample
 	// Initialize sample slot with default values.
 	void Initialize(MODTYPE type = MOD_TYPE_NONE);
 
+	// Copies sample data from another sample slot and ensures that the 16-bit/stereo flags are set accordingly.
+	bool CopyWaveform(const ModSample &smpFrom);
+
 	// Allocate sample based on a ModSample's properties.
 	// Returns number of bytes allocated, 0 on failure.
 	size_t AllocateSample();
@@ -138,21 +143,31 @@ struct ModSample
 	// Update loop wrap-around buffer
 	void PrecomputeLoops(CSoundFile &sndFile, bool updateChannels = true);
 
+	constexpr bool HasLoop() const noexcept { return uFlags[CHN_LOOP] && nLoopEnd > nLoopStart; }
+	constexpr bool HasSustainLoop() const noexcept { return uFlags[CHN_SUSTAINLOOP] && nSustainEnd > nSustainStart; }
+	constexpr bool HasPingPongLoop() const noexcept { return uFlags.test_all(CHN_LOOP | CHN_PINGPONGLOOP) && nLoopEnd > nLoopStart; }
+	constexpr bool HasPingPongSustainLoop() const noexcept { return uFlags.test_all(CHN_SUSTAINLOOP | CHN_PINGPONGSUSTAIN) && nSustainEnd > nSustainStart; }
+
 	// Remove loop points if they're invalid.
 	void SanitizeLoops();
 
 	// Transpose <-> Frequency conversions
 	static uint32 TransposeToFrequency(int transpose, int finetune = 0);
 	void TransposeToFrequency();
-	static int FrequencyToTranspose(uint32 freq);
+	static std::pair<int8, int8> FrequencyToTranspose(uint32 freq);
 	void FrequencyToTranspose();
 
 	// Transpose the sample by amount specified in octaves (i.e. amount=1 transposes one octave up)
 	void Transpose(double amount);
 
+	// Check if the sample has any valid cue points
+	bool HasAnyCuePoints() const;
 	// Check if the sample's cue points are the default cue point set.
 	bool HasCustomCuePoints() const;
 	void SetDefaultCuePoints();
+	// Set cue points so that they are suitable for regular offset command extension
+	void Set16BitCuePoints();
+	void RemoveAllCuePoints();
 
 	void SetAdlib(bool enable, OPLPatch patch = OPLPatch{{}});
 };
