@@ -20,20 +20,39 @@
 #include "Tagging.h"
 #include "Loaders.h"
 #include "WAVTools.h"
-#include "ChunkReader.h"
+#include "../common/FileReader.h"
 #include "modsmp_ctrl.h"
-#include "../soundbase/SampleFormatConverters.h"
-#include "../soundbase/SampleFormatCopy.h"
+#include "openmpt/soundbase/Copy.hpp"
+#include "openmpt/soundbase/SampleConvert.hpp"
+#include "openmpt/soundbase/SampleDecode.hpp"
+#include "../soundlib/SampleCopy.h"
 #include "../soundlib/ModSampleCopy.h"
-//#include "../common/mptCRC.h"
+#include "mpt/io/base.hpp"
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_stdstream.hpp"
+//#include "mpt/crc/crc.hpp"
 #include "OggStream.h"
 #ifdef MPT_WITH_OGG
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
+#endif // MPT_COMPILER_CLANG
 #include <ogg/ogg.h>
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic pop
+#endif // MPT_COMPILER_CLANG
 #endif // MPT_WITH_OGG
 #ifdef MPT_WITH_FLAC
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
+#endif // MPT_COMPILER_CLANG
 #include <FLAC/stream_decoder.h>
 #include <FLAC/stream_encoder.h>
 #include <FLAC/metadata.h>
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic pop
+#endif // MPT_COMPILER_CLANG
 #endif // MPT_WITH_FLAC
 
 
@@ -61,7 +80,7 @@ struct FLACDecoder
 		{
 			FileReader::off_t readBytes = *bytes;
 			LimitMax(readBytes, file.BytesLeft());
-			file.ReadRaw(buffer, readBytes);
+			file.ReadRaw(mpt::byte_cast<mpt::byte_span>(mpt::span(buffer, readBytes)));
 			*bytes = readBytes;
 			if(*bytes == 0)
 				return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
@@ -167,7 +186,7 @@ struct FLACDecoder
 		{
 			// Init sample information
 			client.sndFile.DestroySampleThreadsafe(client.sample);
-			strcpy(client.sndFile.m_szNames[client.sample], "");
+			client.sndFile.m_szNames[client.sample] = "";
 			sample.Initialize();
 			sample.uFlags.set(CHN_16BIT, metadata->data.stream_info.bits_per_sample > 8);
 			sample.uFlags.set(CHN_STEREO, metadata->data.stream_info.channels > 1);
@@ -178,8 +197,8 @@ struct FLACDecoder
 		} else if(metadata->type == FLAC__METADATA_TYPE_APPLICATION && !memcmp(metadata->data.application.id, "riff", 4) && client.ready)
 		{
 			// Try reading RIFF loop points and other sample information
-			ChunkReader data(mpt::as_span(metadata->data.application.data, metadata->length));
-			ChunkReader::ChunkList<RIFFChunk> chunks = data.ReadChunks<RIFFChunk>(2);
+			FileReader data(mpt::as_span(metadata->data.application.data, metadata->length));
+			FileReader::ChunkList<RIFFChunk> chunks = data.ReadChunks<RIFFChunk>(2);
 
 			// We're not really going to read a WAV file here because there will be only one RIFF chunk per metadata event, but we can still re-use the code for parsing RIFF metadata...
 			WAVReader riffReader(data);
@@ -195,9 +214,7 @@ struct FLACDecoder
 				const FLAC__uint32 length = metadata->data.vorbis_comment.comments[i].length;
 				if(length > 6 && !mpt::CompareNoCaseAscii(tag, "TITLE=", 6))
 				{
-					mpt::ustring sampleName;
-					mpt::String::Read<mpt::String::maybeNullTerminated>(sampleName, mpt::CharsetUTF8, tag + 6, length - 6);
-					mpt::String::Copy(client.sndFile.m_szNames[client.sample], mpt::ToCharset(client.sndFile.GetCharsetInternal(), sampleName));
+					client.sndFile.m_szNames[client.sample] = mpt::ToCharset(client.sndFile.GetCharsetInternal(), mpt::Charset::UTF8, mpt::String::ReadBuf(mpt::String::maybeNullTerminated, tag + 6, length - 6));
 				} else if(length > 11 && !mpt::CompareNoCaseAscii(tag, "SAMPLERATE=", 11))
 				{
 					uint32 sampleRate = ConvertStrTo<uint32>(tag + 11);
@@ -246,7 +263,7 @@ bool CSoundFile::ReadFLACSample(SAMPLEINDEX sample, FileReader &file)
 		file.Rewind();
 		bool oggOK = false;
 		bool needMoreData = true;
-		static const long bufsize = 65536;
+		constexpr long bufsize = 65536;
 		std::size_t readSize = 0;
 		char *buf = nullptr;
 		ogg_sync_state oy;
@@ -275,7 +292,7 @@ bool CSoundFile::ReadFLACSample(SAMPLEINDEX sample, FileReader &file)
 				needMoreData = false;
 				break;
 			}
-			readSize = file.ReadRaw(buf, bufsize);
+			readSize = file.ReadRaw(mpt::span(buf, bufsize)).size();
 			if(ogg_sync_wrote(&oy, static_cast<long>(readSize)) != 0)
 			{
 				oggOK = false;
@@ -487,7 +504,7 @@ struct FLAC__StreamEncoder_RAII
 	{
 		mpt::ofstream & file = *reinterpret_cast<mpt::ofstream*>(client_data);
 		MPT_UNUSED_VARIABLE(encoder);
-		if(!Util::TypeCanHoldValue<mpt::IO::Offset>(absolute_byte_offset))
+		if(!mpt::in_range<mpt::IO::Offset>(absolute_byte_offset))
 		{
 			return FLAC__STREAM_ENCODER_SEEK_STATUS_ERROR;
 		}
@@ -506,7 +523,7 @@ struct FLAC__StreamEncoder_RAII
 		{
 			return FLAC__STREAM_ENCODER_TELL_STATUS_ERROR;
 		}
-		if(!mpt::IO::OffsetFits<FLAC__uint64>(pos))
+		if(!mpt::in_range<FLAC__uint64>(pos))
 		{
 			return FLAC__STREAM_ENCODER_TELL_STATUS_ERROR;
 		}
@@ -539,13 +556,14 @@ public:
 bool CSoundFile::SaveFLACSample(SAMPLEINDEX nSample, std::ostream &f) const
 {
 #ifdef MPT_WITH_FLAC
+	const ModSample &sample = Samples[nSample];
+	if(sample.uFlags[CHN_ADLIB])
+		return false;
+
 	FLAC__StreamEncoder_RAII encoder(f);
 	if(encoder == nullptr)
-	{
 		return false;
-	}
 
-	const ModSample &sample = Samples[nSample];
 	uint32 sampleRate = sample.GetSampleRate(GetType());
 
 	// First off, set up all the metadata...
@@ -562,15 +580,15 @@ bool CSoundFile::SaveFLACSample(SAMPLEINDEX nSample, std::ostream &f) const
 	{
 		// Store sample name
 		FLAC__StreamMetadata_VorbisComment_Entry entry;
-		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "TITLE", mpt::ToCharset(mpt::CharsetUTF8, GetCharsetInternal(), m_szNames[nSample]).c_str());
+		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "TITLE", mpt::ToCharset(mpt::Charset::UTF8, GetCharsetInternal(), m_szNames[nSample]).c_str());
 		FLAC__metadata_object_vorbiscomment_append_comment(metadata[0], entry, false);
-		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "ENCODER", mpt::ToCharset(mpt::CharsetUTF8, Version::Current().GetOpenMPTVersionString()).c_str());
+		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "ENCODER", mpt::ToCharset(mpt::Charset::UTF8, Version::Current().GetOpenMPTVersionString()).c_str());
 		FLAC__metadata_object_vorbiscomment_append_comment(metadata[0], entry, false);
 		if(sampleRate > FLAC__MAX_SAMPLE_RATE)
 		{
 			// FLAC only supports a sample rate of up to 655350 Hz.
 			// Store the real sample rate in a custom Vorbis comment.
-			FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "SAMPLERATE", mpt::fmt::val(sampleRate).c_str());
+			FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "SAMPLERATE", mpt::afmt::val(sampleRate).c_str());
 			FLAC__metadata_object_vorbiscomment_append_comment(metadata[0], entry, false);
 		}
 	}
@@ -636,14 +654,14 @@ bool CSoundFile::SaveFLACSample(SAMPLEINDEX nSample, std::ostream &f) const
 		{
 			RIFFChunk header;
 			uint32le numPoints;
-			WAVCuePoint cues[CountOf(sample.cues)];
-		} chunk;
+			WAVCuePoint cues[mpt::array_size<decltype(sample.cues)>::size];
+		} chunk{};
 
 		chunk.header.id = RIFFChunk::idcue_;
 		chunk.header.length = 4 + sizeof(chunk.cues);
-		chunk.numPoints = CountOf(sample.cues);
+		chunk.numPoints = mpt::saturate_cast<uint32>(std::size(sample.cues));
 
-		for(uint32 i = 0; i < CountOf(sample.cues); i++)
+		for(uint32 i = 0; i < std::size(sample.cues); i++)
 		{
 			chunk.cues[i].ConvertToWAV(i, sample.cues[i]);
 		}
@@ -679,7 +697,7 @@ bool CSoundFile::SaveFLACSample(SAMPLEINDEX nSample, std::ostream &f) const
 	FLAC__int32 buffer[mpt::IO::BUFFERSIZE_TINY];
 	while(framesRemain && success)
 	{
-		const SmpLength copyFrames = std::min(framesRemain, mpt::saturate_cast<SmpLength>(mpt::size(buffer) / numChannels));
+		const SmpLength copyFrames = std::min(framesRemain, mpt::saturate_cast<SmpLength>(std::size(buffer) / numChannels));
 
 		// First, convert to a 32-bit integer buffer
 		switch(sample.GetElementarySampleSize())
@@ -690,7 +708,7 @@ bool CSoundFile::SaveFLACSample(SAMPLEINDEX nSample, std::ostream &f) const
 		}
 
 		// Now do the actual encoding
-		success = FLAC__stream_encoder_process_interleaved(encoder, buffer, copyFrames) != false;
+		success = FLAC__stream_encoder_process_interleaved(encoder, buffer, copyFrames) != static_cast<FLAC__bool>(false);
 
 		framesRead += copyFrames;
 		framesRemain -= copyFrames;
