@@ -13,23 +13,23 @@
 #include "stdafx.h"
 #include "Loaders.h"
 #include "SampleIO.h"
-#include "../soundbase/SampleFormatConverters.h"
-#include "../soundbase/SampleFormatCopy.h"
+#include "openmpt/soundbase/SampleDecode.hpp"
+#include "SampleCopy.h"
+#include "SampleNormalize.h"
 #include "ModSampleCopy.h"
 #include "ITCompression.h"
-#include "../common/mptIO.h"
 #ifndef MODPLUG_NO_FILESAVE
 #include "../common/mptFileIO.h"
+#include "mpt/io/base.hpp"
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_stdstream.hpp"
+#include "mpt/io_write/buffer.hpp"
 #endif
 #include "BitReader.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
 
-// Sample decompression routines in other source files
-void AMSUnpack(const int8 * const source, size_t sourceSize, void * const dest, const size_t destSize, char packCharacter);
-uintptr_t DMFUnpack(FileReader &file, uint8 *psample, uint32 maxlen);
-
 
 // Read a sample from memory
 size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
@@ -45,7 +45,7 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 
 	FileReader::off_t filePosition = file.GetPosition();
 	const std::byte * sourceBuf = nullptr;
-	FileReader::PinnedRawDataView restrictedSampleDataView;
+	FileReader::PinnedView restrictedSampleDataView;
 	FileReader::off_t fileSize = 0;
 	if(UsesFileReaderForDecoding())
 	{
@@ -53,9 +53,11 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		fileSize = file.BytesLeft();
 	} else if(!IsVariableLengthEncoded())
 	{
-		restrictedSampleDataView = file.GetPinnedRawDataView(CalculateEncodedSize(sample.nLength));
+		restrictedSampleDataView = file.GetPinnedView(CalculateEncodedSize(sample.nLength));
 		sourceBuf = restrictedSampleDataView.data();
 		fileSize = restrictedSampleDataView.size();
+		if(sourceBuf == nullptr)
+			return 0;
 	} else
 	{
 		MPT_ASSERT_NOTREACHED();
@@ -173,11 +175,11 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		int8 packCharacter = file.ReadUint8();
 		bytesRead += 9;
 
-		FileReader::PinnedRawDataView packedDataView = file.ReadPinnedRawDataView(sourceSize);
+		FileReader::PinnedView packedDataView = file.ReadPinnedView(sourceSize);
 		LimitMax(sourceSize, mpt::saturate_cast<uint32>(packedDataView.size()));
 		bytesRead += sourceSize;
 
-		AMSUnpack(reinterpret_cast<const int8 *>(packedDataView.data()), packedDataView.size(), sample.samplev(), sample.GetSampleSizeInBytes(), packCharacter);
+		AMSUnpack(packedDataView.span(), mpt::as_span(sample.sampleb(), sample.GetSampleSizeInBytes()), packCharacter);
 		if(sample.uFlags[CHN_16BIT] && !mpt::endian_is_little())
 		{
 			auto p = sample.sample16();
@@ -248,96 +250,31 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		{
 			bytesRead = DMFUnpack(file, mpt::byte_cast<uint8 *>(sample.sampleb()), sample.GetSampleSizeInBytes());
 		}
-#ifdef MODPLUG_TRACKER
 	} else if((GetEncoding() == uLaw || GetEncoding() == aLaw) && GetBitDepth() == 16 && (GetChannelFormat() == mono || GetChannelFormat() == stereoInterleaved))
 	{
-		// 8-to-16 bit G.711 u-law / a-law
-		static constexpr int16 uLawTable[256] =
-		{
-			-32124,-31100,-30076,-29052,-28028,-27004,-25980,-24956,
-			-23932,-22908,-21884,-20860,-19836,-18812,-17788,-16764,
-			-15996,-15484,-14972,-14460,-13948,-13436,-12924,-12412,
-			-11900,-11388,-10876,-10364, -9852, -9340, -8828, -8316,
-			 -7932, -7676, -7420, -7164, -6908, -6652, -6396, -6140,
-			 -5884, -5628, -5372, -5116, -4860, -4604, -4348, -4092,
-			 -3900, -3772, -3644, -3516, -3388, -3260, -3132, -3004,
-			 -2876, -2748, -2620, -2492, -2364, -2236, -2108, -1980,
-			 -1884, -1820, -1756, -1692, -1628, -1564, -1500, -1436,
-			 -1372, -1308, -1244, -1180, -1116, -1052,  -988,  -924,
-			  -876,  -844,  -812,  -780,  -748,  -716,  -684,  -652,
-			  -620,  -588,  -556,  -524,  -492,  -460,  -428,  -396,
-			  -372,  -356,  -340,  -324,  -308,  -292,  -276,  -260,
-			  -244,  -228,  -212,  -196,  -180,  -164,  -148,  -132,
-			  -120,  -112,  -104,   -96,   -88,   -80,   -72,   -64,
-			   -56,   -48,   -40,   -32,   -24,   -16,    -8,    -1,
-			 32124, 31100, 30076, 29052, 28028, 27004, 25980, 24956,
-			 23932, 22908, 21884, 20860, 19836, 18812, 17788, 16764,
-			 15996, 15484, 14972, 14460, 13948, 13436, 12924, 12412,
-			 11900, 11388, 10876, 10364,  9852,  9340,  8828,  8316,
-			  7932,  7676,  7420,  7164,  6908,  6652,  6396,  6140,
-			  5884,  5628,  5372,  5116,  4860,  4604,  4348,  4092,
-			  3900,  3772,  3644,  3516,  3388,  3260,  3132,  3004,
-			  2876,  2748,  2620,  2492,  2364,  2236,  2108,  1980,
-			  1884,  1820,  1756,  1692,  1628,  1564,  1500,  1436,
-			  1372,  1308,  1244,  1180,  1116,  1052,   988,   924,
-			   876,   844,   812,   780,   748,   716,   684,   652,
-			   620,   588,   556,   524,   492,   460,   428,   396,
-			   372,   356,   340,   324,   308,   292,   276,   260,
-			   244,   228,   212,   196,   180,   164,   148,   132,
-			   120,   112,   104,    96,    88,    80,    72,    64,
-			    56,    48,    40,    32,    24,    16,     8,     0,
-		};
-
-		static constexpr int16 aLawTable[256] =
-		{
-			 -5504, -5248, -6016, -5760, -4480, -4224, -4992, -4736,
-			 -7552, -7296, -8064, -7808, -6528, -6272, -7040, -6784,
-			 -2752, -2624, -3008, -2880, -2240, -2112, -2496, -2368,
-			 -3776, -3648, -4032, -3904, -3264, -3136, -3520, -3392,
-			-22016,-20992,-24064,-23040,-17920,-16896,-19968,-18944,
-			-30208,-29184,-32256,-31232,-26112,-25088,-28160,-27136,
-			-11008,-10496,-12032,-11520, -8960, -8448, -9984, -9472,
-			-15104,-14592,-16128,-15616,-13056,-12544,-14080,-13568,
-			  -344,  -328,  -376,  -360,  -280,  -264,  -312,  -296,
-			  -472,  -456,  -504,  -488,  -408,  -392,  -440,  -424,
-			   -88,   -72,   -120,  -104,  -24,   -8,    -56,   -40,
-			  -216,  -200,  -248,  -232,  -152,  -136,  -184,  -168,
-			 -1376, -1312, -1504, -1440, -1120, -1056, -1248, -1184,
-			 -1888, -1824, -2016, -1952, -1632, -1568, -1760, -1696,
-			  -688,  -656,  -752,  -720,  -560,  -528,  -624,  -592,
-			  -944,  -912,  -1008, -976,  -816,  -784,  -880,  -848,
-			  5504,  5248,  6016,  5760,  4480,  4224,  4992,  4736,
-			  7552,  7296,  8064,  7808,  6528,  6272,  7040,  6784,
-			  2752,  2624,  3008,  2880,  2240,  2112,  2496,  2368,
-			  3776,  3648,  4032,  3904,  3264,  3136,  3520,  3392,
-			 22016, 20992, 24064, 23040, 17920, 16896, 19968, 18944,
-			 30208, 29184, 32256, 31232, 26112, 25088, 28160, 27136,
-			 11008, 10496, 12032, 11520,  8960,  8448,  9984,  9472,
-			 15104, 14592, 16128, 15616, 13056, 12544, 14080, 13568,
-			   344,   328,   376,   360,   280,   264,   312,   296,
-			   472,   456,   504,   488,   408,   392,   440,   424,
-			    88,    72,   120,   104,    24,     8,    56,    40,
-			   216,   200,   248,   232,   152,   136,   184,   168,
-			  1376,  1312,  1504,  1440,  1120,  1056,  1248,  1184,
-			  1888,  1824,  2016,  1952,  1632,  1568,  1760,  1696,
-			   688,   656,   752,   720,   560,   528,   624,   592,
-			   944,   912,  1008,   976,   816,   784,   880,   848,
-		};
-
-		const auto &lut = GetEncoding() == uLaw ? uLawTable : aLawTable;
 
 		SmpLength readLength = sample.nLength * GetNumChannels();
 		LimitMax(readLength, mpt::saturate_cast<SmpLength>(fileSize));
 		bytesRead = readLength;
 
-		const uint8 *inBuf = mpt::byte_cast<const uint8*>(sourceBuf);
+		const std::byte *inBuf = sourceBuf;
 		int16 *outBuf = sample.sample16();
 
-		while(readLength--)
+		if(GetEncoding() == uLaw)
 		{
-			*(outBuf++) = lut[*(inBuf++)];
+			SC::DecodeInt16uLaw conv;
+			while(readLength--)
+			{
+				*(outBuf++) = conv(inBuf++);
+			}
+		} else
+		{
+			SC::DecodeInt16ALaw conv;
+			while(readLength--)
+			{
+				*(outBuf++) = conv(inBuf++);
+			}
 		}
-#endif // MODPLUG_TRACKER
 	}
 
 
@@ -360,9 +297,6 @@ size_t SampleIO::ReadSample(ModSample &sample, FileReader &file) const
 		case MT2:
 			bytesRead = CopyMonoSample<SC::DecodeInt8Delta>(sample, sourceBuf, fileSize);
 			break;
-		case PCM7to8:		// 7 Bit stored as 8-Bit with highest bit unused / Mono / Signed / PCM
-			bytesRead = CopyMonoSample<SC::DecodeInt7>(sample, sourceBuf, fileSize);
-			break;
 		default:
 			MPT_ASSERT_NOTREACHED();
 			break;
