@@ -1985,6 +1985,44 @@ void CSoundFile::ProcessRamping(ModChannel &chn) const
 }
 
 
+int CSoundFile::HandleNoteChangeFilter(ModChannel &chn) const
+{
+	int cutoff = -1;
+	if(!chn.triggerNote)
+		return cutoff;
+
+	bool useFilter = !m_SongFlags[SONG_MPTFILTERMODE];
+	if(const ModInstrument *pIns = chn.pModInstrument; pIns != nullptr)
+	{
+		if(pIns->IsResonanceEnabled())
+		{
+			chn.nResonance = pIns->GetResonance();
+			useFilter = true;
+		}
+		if(pIns->IsCutoffEnabled())
+		{
+			chn.nCutOff = pIns->GetCutoff();
+			useFilter = true;
+		}
+		if(useFilter && (pIns->filterMode != FilterMode::Unchanged))
+		{
+			chn.nFilterMode = pIns->filterMode;
+		}
+	} else
+	{
+		chn.nVolSwing = chn.nPanSwing = 0;
+		chn.nCutSwing = chn.nResSwing = 0;
+	}
+	if((chn.nCutOff < 0x7F || m_playBehaviour[kITFilterBehaviour]) && useFilter)
+	{
+		cutoff = SetupChannelFilter(chn, true);
+		if(cutoff >= 0)
+			cutoff = chn.nCutOff / 2u;
+	}
+	return cutoff;
+}
+
+
 // Returns channel increment and frequency with FREQ_FRACBITS fractional bits
 std::pair<SamplePosition, uint32> CSoundFile::GetChannelIncrement(const ModChannel &chn, uint32 period, int periodFrac) const
 {
@@ -2251,38 +2289,8 @@ bool CSoundFile::ReadNote()
 		}
 
 		// Setup Initial Filter for this note
-		int cutoff = -1;
-		if(chn.triggerNote)
-		{
-			bool useFilter = !m_SongFlags[SONG_MPTFILTERMODE];
-			if(pIns)
-			{
-				if(pIns->IsResonanceEnabled())
-				{
-					chn.nResonance = pIns->GetResonance();
-					useFilter = true;
-				}
-				if(pIns->IsCutoffEnabled())
-				{
-					chn.nCutOff = pIns->GetCutoff();
-					useFilter = true;
-				}
-				if(useFilter && (pIns->filterMode != FilterMode::Unchanged))
-				{
-					chn.nFilterMode = pIns->filterMode;
-				}
-			} else
-			{
-				chn.nVolSwing = chn.nPanSwing = 0;
-				chn.nCutSwing = chn.nResSwing = 0;
-			}
-			if((chn.nCutOff < 0x7F || m_playBehaviour[kITFilterBehaviour]) && useFilter)
-			{
-				cutoff = SetupChannelFilter(chn, true);
-				if(cutoff >= 0)
-					cutoff = chn.nCutOff / 2u;
-			}
-		}
+		if(int cutoff = HandleNoteChangeFilter(chn); cutoff >= 0 && chn.dwFlags[CHN_ADLIB] && m_opl)
+			m_opl->Volume(nChn, static_cast<uint8>(cutoff), true);
 
 		// Now that all relevant envelopes etc. have been processed, we can parse the MIDI macro data.
 		ProcessMacroOnChannel(nChn);
@@ -2291,14 +2299,11 @@ bool CSoundFile::ReadNote()
 		if(samplePlaying)
 		{
 			int envCutoff = ProcessPitchFilterEnvelope(chn, period);
-			if(envCutoff >= 0)
-				cutoff = envCutoff / 4;
+			// Cutoff doubles as modulator intensity for FM instruments
+			if(envCutoff >= 0 && chn.dwFlags[CHN_ADLIB] && m_opl)
+				m_opl->Volume(nChn, static_cast<uint8>(envCutoff / 4), true);
 		}
 
-		// Cutoff doubles as modulator intensity for FM instruments
-		if(cutoff >= 0 && chn.dwFlags[CHN_ADLIB] && m_opl)
-			m_opl->Volume(nChn, static_cast<uint8>(cutoff), true);
-
 		if(chn.rowCommand.volcmd == VOLCMD_VIBRATODEPTH &&
 			(chn.rowCommand.command == CMD_VIBRATO || chn.rowCommand.command == CMD_VIBRATOVOL || chn.rowCommand.command == CMD_FINEVIBRATO))
 		{
