@@ -14,6 +14,8 @@
 #include "Tagging.h"
 #include "../common/version.h"
 #ifndef MODPLUG_NO_FILESAVE
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_virtual_wrapper.hpp"
 #include "../common/mptFileIO.h"
 #endif
 
@@ -45,11 +47,11 @@ WAVReader::WAVReader(FileReader &inputFile) : file(inputFile)
 
 	auto chunks = file.ReadChunks<RIFFChunk>(2);
 
-	if(chunks.size() >= 4
-		&& chunks[1].GetHeader().GetID() == RIFFChunk::iddata
-		&& chunks[1].GetHeader().GetLength() % 2u != 0
-		&& chunks[2].GetHeader().GetLength() == 0
-		&& chunks[3].GetHeader().GetID() == RIFFChunk::id____)
+	if(chunks.chunks.size() >= 4
+		&& chunks.chunks[1].GetHeader().GetID() == RIFFChunk::iddata
+		&& chunks.chunks[1].GetHeader().GetLength() % 2u != 0
+		&& chunks.chunks[2].GetHeader().GetLength() == 0
+		&& chunks.chunks[3].GetHeader().GetID() == RIFFChunk::id____)
 	{
 		// Houston, we have a problem: Old versions of (Open)MPT didn't write RIFF padding bytes. -_-
 		// Luckily, the only RIFF chunk with an odd size those versions would ever write would be the "data" chunk
@@ -131,7 +133,7 @@ WAVReader::WAVReader(FileReader &inputFile) : file(inputFile)
 }
 
 
-void WAVReader::FindMetadataChunks(ChunkReader::ChunkList<RIFFChunk> &chunks)
+void WAVReader::FindMetadataChunks(FileReader::ChunkList<RIFFChunk> &chunks)
 {
 	// Read sample loop points and other sampler information
 	smplChunk = chunks.GetChunk(RIFFChunk::idsmpl);
@@ -141,7 +143,7 @@ void WAVReader::FindMetadataChunks(ChunkReader::ChunkList<RIFFChunk> &chunks)
 	cueChunk = chunks.GetChunk(RIFFChunk::idcue_);
 
 	// Read text chunks
-	ChunkReader listChunk = chunks.GetChunk(RIFFChunk::idLIST);
+	FileReader listChunk = chunks.GetChunk(RIFFChunk::idLIST);
 	if(listChunk.ReadMagic("INFO"))
 	{
 		infoChunk = listChunk.ReadChunks<RIFFChunk>(2);
@@ -152,7 +154,7 @@ void WAVReader::FindMetadataChunks(ChunkReader::ChunkList<RIFFChunk> &chunks)
 }
 
 
-uint16 WAVReader::GetFileCodePage(ChunkReader::ChunkList<RIFFChunk> &chunks)
+uint16 WAVReader::GetFileCodePage(FileReader::ChunkList<RIFFChunk> &chunks)
 {
 	FileReader csetChunk = chunks.GetChunk(RIFFChunk::idCSET);
 	if(!csetChunk.IsValid())
@@ -162,31 +164,31 @@ uint16 WAVReader::GetFileCodePage(ChunkReader::ChunkList<RIFFChunk> &chunks)
 		{
 			std::string versionString;
 			iSFT.ReadString<mpt::String::maybeNullTerminated>(versionString, iSFT.BytesLeft());
-			versionString = mpt::String::Trim(versionString);
-			Version version = Version::Parse(mpt::ToUnicode(mpt::CharsetISO8859_1, versionString));
-			if(version && version < MAKE_VERSION_NUMERIC(1,28,00,02))
+			versionString = mpt::trim(versionString);
+			Version version = Version::Parse(mpt::ToUnicode(mpt::Charset::ISO8859_1, versionString));
+			if(version && version < MPT_V("1.28.00.02"))
 			{
-				return 1252; // mpt::CharsetWindows1252; // OpenMPT up to and including 1.28.00.01 wrote metadata in windows-1252 encoding
+				return 1252; // mpt::Charset::Windows1252; // OpenMPT up to and including 1.28.00.01 wrote metadata in windows-1252 encoding
 			} else
 			{
-				return 28591; // mpt::CharsetISO8859_1; // as per spec
+				return 28591; // mpt::Charset::ISO8859_1; // as per spec
 			}
 		} else
 		{
-			return 28591; // mpt::CharsetISO8859_1; // as per spec
+			return 28591; // mpt::Charset::ISO8859_1; // as per spec
 		}
 	}
 	if(!csetChunk.CanRead(2))
 	{
 		// chunk not parsable
-		return 28591; // mpt::CharsetISO8859_1;
+		return 28591; // mpt::Charset::ISO8859_1;
 	}
 	uint16 codepage = csetChunk.ReadUint16LE();
 	return codepage;
 }
 
 
-void WAVReader::ApplySampleSettings(ModSample &sample, mpt::Charset sampleCharset, char (&sampleName)[MAX_SAMPLENAME])
+void WAVReader::ApplySampleSettings(ModSample &sample, mpt::Charset sampleCharset, mpt::charbuf<MAX_SAMPLENAME> &sampleName)
 {
 	// Read sample name
 	FileReader textChunk = infoChunk.GetChunk(RIFFChunk::idINAM);
@@ -194,12 +196,12 @@ void WAVReader::ApplySampleSettings(ModSample &sample, mpt::Charset sampleCharse
 	{
 		std::string sampleNameEncoded;
 		textChunk.ReadString<mpt::String::nullTerminated>(sampleNameEncoded, textChunk.GetLength());
-		mpt::String::Copy(sampleName, mpt::ToCharset(sampleCharset, mpt::ToUnicode(codePage, mpt::CharsetWindows1252, sampleNameEncoded)));
+		sampleName = mpt::ToCharset(sampleCharset, mpt::ToUnicode(codePage, mpt::Charset::Windows1252, sampleNameEncoded));
 	}
 	if(isDLS)
 	{
 		// DLS sample -> sample filename
-		mpt::String::Copy(sample.filename, sampleName);
+		sample.filename = sampleName;
 	}
 
 	// Read software name
@@ -248,13 +250,14 @@ void WAVReader::ApplySampleSettings(ModSample &sample, mpt::Charset sampleCharse
 	if(cueChunk.IsValid())
 	{
 		uint32 numPoints = cueChunk.ReadUint32LE();
-		LimitMax(numPoints, mpt::saturate_cast<uint32>(MPT_ARRAY_COUNT(sample.cues)));
+		LimitMax(numPoints, mpt::saturate_cast<uint32>(std::size(sample.cues)));
 		for(uint32 i = 0; i < numPoints; i++)
 		{
 			WAVCuePoint cuePoint;
 			cueChunk.ReadStruct(cuePoint);
 			sample.cues[i] = cuePoint.position;
 		}
+		std::fill(std::begin(sample.cues) + numPoints, std::end(sample.cues), MAX_SAMPLE_LENGTH);
 	}
 
 	// Read MPT extra info
@@ -264,9 +267,9 @@ void WAVReader::ApplySampleSettings(ModSample &sample, mpt::Charset sampleCharse
 	{
 		if(mptInfo.flags & WAVExtraChunk::setPanning) sample.uFlags.set(CHN_PANNING);
 
-		sample.nPan = std::min<uint16>(mptInfo.defaultPan, 256);
-		sample.nVolume = std::min<uint16>(mptInfo.defaultVolume, 256);
-		sample.nGlobalVol = std::min<uint16>(mptInfo.globalVolume, 64);
+		sample.nPan = std::min(static_cast<uint16>(mptInfo.defaultPan), uint16(256));
+		sample.nVolume = std::min(static_cast<uint16>(mptInfo.defaultVolume), uint16(256));
+		sample.nGlobalVol = std::min(static_cast<uint16>(mptInfo.globalVolume), uint16(64));
 		sample.nVibType = static_cast<VibratoType>(mptInfo.vibratoType.get());
 		sample.nVibSweep = mptInfo.vibratoSweep;
 		sample.nVibDepth = mptInfo.vibratoDepth;
@@ -335,45 +338,34 @@ void WAVSampleLoop::ConvertToWAV(SmpLength start, SmpLength end, bool bidi)
 
 
 // Output to stream: Initialize with std::ostream*.
-WAVWriter::WAVWriter(std::ostream *stream) : s(stream)
+WAVWriter::WAVWriter(mpt::IO::OFileBase &stream)
+	: s(stream)
 {
 	// Skip file header for now
 	Seek(sizeof(RIFFHeader));
 }
 
 
-// Output to clipboard: Initialize with pointer to memory and size of reserved memory.
-WAVWriter::WAVWriter(mpt::byte_span data) : memory(data)
+WAVWriter::~WAVWriter()
 {
-	// Skip file header for now
-	Seek(sizeof(RIFFHeader));
-}
-
-
-WAVWriter::~WAVWriter() noexcept(false)
-{
-	if(!s || s->good())
-	{
-		Finalize();
-	}
+	MPT_ASSERT(finalized);
 }
 
 
 // Finalize the file by closing the last open chunk and updating the file header. Returns total size of file.
-size_t WAVWriter::Finalize()
+std::size_t WAVWriter::Finalize()
 {
 	FinalizeChunk();
 
 	RIFFHeader fileHeader;
+	Clear(fileHeader);
 	fileHeader.magic = RIFFHeader::idRIFF;
 	fileHeader.length = static_cast<uint32>(totalSize - 8);
 	fileHeader.type = RIFFHeader::idWAVE;
 
 	Seek(0);
 	Write(fileHeader);
-
-	s = nullptr;
-	memory = {};
+	finalized = true;
 
 	return totalSize;
 }
@@ -395,10 +387,10 @@ void WAVWriter::FinalizeChunk()
 {
 	if(chunkStartPos != 0)
 	{
-		const size_t chunkSize = position - (chunkStartPos + sizeof(RIFFChunk));
+		const std::size_t chunkSize = position - (chunkStartPos + sizeof(RIFFChunk));
 		chunkHeader.length = mpt::saturate_cast<uint32>(chunkSize);
 
-		size_t curPos = position;
+		std::size_t curPos = position;
 		Seek(chunkStartPos);
 		Write(chunkHeader);
 
@@ -416,36 +408,43 @@ void WAVWriter::FinalizeChunk()
 
 
 // Seek to a position in file.
-void WAVWriter::Seek(size_t pos)
+void WAVWriter::Seek(std::size_t pos)
 {
 	position = pos;
 	totalSize = std::max(totalSize, position);
-
-	if(s != nullptr)
-	{
-		s->seekp(position);
-	}
+	mpt::IO::SeekAbsolute(s, pos);
 }
 
 
 // Write some data to the file.
-void WAVWriter::Write(const void *data, size_t numBytes)
+void WAVWriter::Write(mpt::const_byte_span data)
 {
-	if(s != nullptr)
+	MPT_ASSERT(!finalized);
+	auto success = mpt::IO::WriteRaw(s, data);
+	MPT_ASSERT(success); // this assertion is useful to catch mis-calculation of required buffer size for pre-allocate in-memory file buffers (like in View_smp.cpp for clipboard)
+	if(!success)
 	{
-		s->write(static_cast<const char*>(data), numBytes);
-	} else if(!memory.empty())
-	{
-		if(position <= memory.size() && numBytes <= memory.size() - position)
-		{
-			memcpy(memory.data() + position, data, numBytes);
-		} else
-		{
-			// Should never happen - did we calculate a wrong memory size?
-			MPT_ASSERT_NOTREACHED();
-		}
+		return;
 	}
-	position += numBytes;
+	position += data.size();
+	totalSize = std::max(totalSize, position);
+}
+
+
+void WAVWriter::WriteBeforeDirect()
+{
+	MPT_ASSERT(!finalized);
+}
+
+
+void WAVWriter::WriteAfterDirect(bool success, std::size_t count)
+{
+	MPT_ASSERT(success); // this assertion is useful to catch mis-calculation of required buffer size for pre-allocate in-memory file buffers (like in View_smp.cpp for clipboard)
+	if (!success)
+	{
+		return;
+	}
+	position += count;
 	totalSize = std::max(totalSize, position);
 }
 
@@ -455,6 +454,7 @@ void WAVWriter::WriteFormat(uint32 sampleRate, uint16 bitDepth, uint16 numChanne
 {
 	StartChunk(RIFFChunk::idfmt_);
 	WAVFormatChunk wavFormat;
+	Clear(wavFormat);
 
 	bool extensible = (numChannels > 2);
 
@@ -470,6 +470,7 @@ void WAVWriter::WriteFormat(uint32 sampleRate, uint16 bitDepth, uint16 numChanne
 	if(extensible)
 	{
 		WAVFormatChunkExtension extFormat;
+		Clear(extFormat);
 		extFormat.size = sizeof(WAVFormatChunkExtension) - sizeof(uint16);
 		extFormat.validBitsPerSample = bitDepth;
 		switch(numChannels)
@@ -507,7 +508,7 @@ void WAVWriter::WriteMetatags(const FileTags &tags)
 
 	StartChunk(RIFFChunk::idLIST);
 	const char info[] = { 'I', 'N', 'F', 'O' };
-	WriteArray(info);
+	Write(info);
 
 	WriteTag(RIFFChunk::idINAM, tags.title);
 	WriteTag(RIFFChunk::idIART, tags.artist);
@@ -525,17 +526,18 @@ void WAVWriter::WriteMetatags(const FileTags &tags)
 // Write a single tag into a open idLIST chunk
 void WAVWriter::WriteTag(RIFFChunk::ChunkIdentifiers id, const mpt::ustring &utext)
 {
-	std::string text = mpt::ToCharset(mpt::CharsetUTF8, utext);
+	std::string text = mpt::ToCharset(mpt::Charset::UTF8, utext);
 	text = text.substr(0, uint32_max - 1u);
 	if(!text.empty())
 	{
 		const uint32 length = mpt::saturate_cast<uint32>(text.length() + 1);
 
 		RIFFChunk chunk;
+		Clear(chunk);
 		chunk.id = static_cast<uint32>(id);
 		chunk.length = length;
 		Write(chunk);
-		Write(text.c_str(), length);
+		Write(mpt::byte_cast<mpt::const_byte_span>(mpt::span(text.c_str(), length)));
 
 		if((length % 2u) != 0)
 		{
@@ -567,6 +569,7 @@ void WAVWriter::WriteLoopInformation(const ModSample &sample)
 
 	// Set up loops
 	WAVSampleLoop loops[2];
+	Clear(loops);
 	if(sample.uFlags[CHN_SUSTAINLOOP])
 	{
 		loops[info.numLoops++].ConvertToWAV(sample.nSustainStart, sample.nSustainEnd, sample.uFlags[CHN_PINGPONGSUSTAIN]);
@@ -593,15 +596,24 @@ void WAVWriter::WriteLoopInformation(const ModSample &sample)
 // Write a sample's cue points to the file.
 void WAVWriter::WriteCueInformation(const ModSample &sample)
 {
-	StartChunk(RIFFChunk::idcue_);
+	uint32 numMarkers = 0;
+	for(const auto cue : sample.cues)
 	{
-		Write(mpt::as_le(static_cast<uint32>(CountOf(sample.cues))));
+		if(cue < sample.nLength)
+			numMarkers++;
 	}
-	for(uint32 i = 0; i < CountOf(sample.cues); i++)
+
+	StartChunk(RIFFChunk::idcue_);
+	Write(mpt::as_le(numMarkers));
+	uint32 i = 0;
+	for(const auto cue : sample.cues)
 	{
-		WAVCuePoint cuePoint;
-		cuePoint.ConvertToWAV(i, sample.cues[i]);
-		Write(cuePoint);
+		if(cue < sample.nLength)
+		{
+			WAVCuePoint cuePoint;
+			cuePoint.ConvertToWAV(i++, cue);
+			Write(cuePoint);
+		}
 	}
 }
 
@@ -624,12 +636,12 @@ void WAVWriter::WriteExtraInformation(const ModSample &sample, MODTYPE modType,
 		// also specify encoding.
 
 		char name[MAX_SAMPLENAME];
-		mpt::String::Write<mpt::String::nullTerminated>(name, sampleName, MAX_SAMPLENAME);
-		WriteArray(name);
+		mpt::String::WriteBuf(mpt::String::nullTerminated, name) = sampleName;
+		Write(name);
 
 		char filename[MAX_SAMPLEFILENAME];
-		mpt::String::Write<mpt::String::nullTerminated>(filename, sample.filename);
-		WriteArray(filename);
+		mpt::String::WriteBuf(mpt::String::nullTerminated, filename) = sample.filename;
+		Write(filename);
 	}
 }
 
