@@ -200,10 +200,12 @@ void IMixPlugin::RecalculateGain()
 }
 
 
-void IMixPlugin::SetDryRatio(uint32 param)
+void IMixPlugin::SetDryRatio(float dryRatio)
 {
-	param = std::min(param, uint32(127));
-	m_pMixStruct->fDryRatio = 1.0f - (param / 127.0f);
+	m_pMixStruct->fDryRatio = std::clamp(dryRatio, 0.0f, 1.0f);
+#ifdef MODPLUG_TRACKER
+	m_SndFile.m_pluginDryWetRatioChanged.set(m_nSlot);
+#endif  // MODPLUG_TRACKER
 }
 
 
@@ -213,7 +215,7 @@ void IMixPlugin::Bypass(bool bypass)
 
 #ifdef MODPLUG_TRACKER
 	if(m_SndFile.GetpModDoc())
-		m_SndFile.GetpModDoc()->UpdateAllViews(nullptr, PluginHint(m_nSlot + 1).Info(), nullptr);
+		m_SndFile.GetpModDoc()->UpdateAllViews(PluginHint(m_nSlot + 1).Info());
 #endif // MODPLUG_TRACKER
 }
 
@@ -497,7 +499,7 @@ void IMixPlugin::SaveAllParameters()
 	m_pMixStruct->defaultProgram = -1;
 	
 	// Default implementation: Save all parameter values
-	PlugParamIndex numParams = std::min(GetNumParameters(), static_cast<int32>((std::numeric_limits<uint32>::max() - sizeof(uint32)) / sizeof(IEEE754binary32LE)));
+	PlugParamIndex numParams = std::min(GetNumParameters(), static_cast<PlugParamIndex>((std::numeric_limits<uint32>::max() - sizeof(uint32)) / sizeof(IEEE754binary32LE)));
 	uint32 nLen = numParams * sizeof(IEEE754binary32LE);
 	if (!nLen) return;
 	nLen += sizeof(uint32);
@@ -882,7 +884,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 	bool bankChanged = (channel.currentBank != midiBank) && (midiBank < 0x4000);
 	bool progChanged = (channel.currentProgram != midiProg) && (midiProg < 0x80);
 	//get vol in [0,128[
-	uint8 volume = static_cast<uint8>(std::min(vol / 2u, 127u));
+	uint8 volume = static_cast<uint8>(std::min((vol + 1u) / 2u, 127u));
 
 	// Bank change
 	if(bankChanged)
