@@ -37,8 +37,8 @@ Echo::Echo(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN *mixStruct)
 {
 	m_param[kEchoWetDry] = 0.5f;
 	m_param[kEchoFeedback] = 0.5f;
-	m_param[kEchoLeftDelay] = 0.25f;
-	m_param[kEchoRightDelay] = 0.25f;
+	m_param[kEchoLeftDelay] = (500.0f - 1.0f) / 1999.0f;
+	m_param[kEchoRightDelay] = (500.0f - 1.0f) / 1999.0f;
 	m_param[kEchoPanDelay] = 0.0f;
 
 	m_mixBuffer.Initialize(2, 2);
@@ -71,7 +71,7 @@ void Echo::Process(float *pOutL, float *pOutR, uint32 numFrames)
 			chnOutput += chnDelay * m_param[kEchoFeedback];
 
 			// Prevent denormals
-			if(mpt::abs(chnOutput) < 1e-24f)
+			if(std::abs(chnOutput) < 1e-24f)
 				chnOutput = 0.0f;
 
 			m_delayLine[m_writePos * 2 + channel] = chnOutput;
@@ -101,7 +101,7 @@ void Echo::SetParameter(PlugParamIndex index, PlugParamValue value)
 {
 	if(index < kEchoNumParameters)
 	{
-		Limit(value, 0.0f, 1.0f);
+		value = mpt::safe_clamp(value, 0.0f, 1.0f);
 		if(index == kEchoPanDelay)
 			value = mpt::round(value);
 		m_param[index] = value;
@@ -125,9 +125,9 @@ void Echo::PositionChanged()
 	try
 	{
 		m_delayLine.assign(m_bufferSize * 2, 0);
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		m_bufferSize = 0;
 	}
 	m_writePos = 0;
@@ -152,9 +152,16 @@ CString Echo::GetParamName(PlugParamIndex param)
 
 CString Echo::GetParamLabel(PlugParamIndex param)
 {
-	if(param == kEchoLeftDelay || param == kEchoRightDelay)
+	switch(param)
+	{
+	case kEchoFeedback:
+		return _T("%");
+	case kEchoLeftDelay:
+	case kEchoRightDelay:
 		return _T("ms");
-	return CString();
+	default:
+		return CString{};
+	}
 }
 
 
@@ -164,12 +171,14 @@ CString Echo::GetParamDisplay(PlugParamIndex param)
 	switch(param)
 	{
 	case kEchoWetDry:
+		s.Format(_T("%.1f : %.1f"), m_param[param] * 100.0f, 100.0f - m_param[param] * 100.0f);
+		break;
 	case kEchoFeedback:
 		s.Format(_T("%.2f"), m_param[param] * 100.0f);
 		break;
 	case kEchoLeftDelay:
 	case kEchoRightDelay:
-		s.Format(_T("%.2f"), m_param[param] * 2000.0f);
+		s.Format(_T("%.2f"), 1.0f + m_param[param] * 1999.0f);
 		break;
 	case kEchoPanDelay:
 		s = (m_param[param] <= 0.5) ? _T("No") : _T("Yes");
@@ -183,8 +192,8 @@ CString Echo::GetParamDisplay(PlugParamIndex param)
 void Echo::RecalculateEchoParams()
 {
 	m_initialFeedback = std::sqrt(1.0f - (m_param[kEchoFeedback] * m_param[kEchoFeedback]));
-	m_delayTime[0] = static_cast<uint32>(m_param[kEchoLeftDelay] * (2 * m_sampleRate));
-	m_delayTime[1] = static_cast<uint32>(m_param[kEchoRightDelay] * (2 * m_sampleRate));
+	m_delayTime[0] = static_cast<uint32>((1.0f + m_param[kEchoLeftDelay] * 1999.0f) / 1000.0f * m_sampleRate);
+	m_delayTime[1] = static_cast<uint32>((1.0f + m_param[kEchoRightDelay] * 1999.0f) / 1000.0f * m_sampleRate);
 	m_crossEcho = (m_param[kEchoPanDelay]) > 0.5f;
 }
 
